{--------------------------------------------------------}
{
{    Rotinas de controle de soletrao Braille
{
{    Autor: Patrick Barboza
{
{    Em 17/05/2023
{
{    Alteraes: 19/05/2023, 20/05/2023, 24/05/2023
{       25/05/2023
{
{--------------------------------------------------------}

unit edBraille;

interface

uses dvCrt, dvWin, edVars, edMensag;

procedure trocaModoSoletrar;
procedure soletraPontosBraille(c: char; modo: integer);

const
    SOLETRABRAILLE = 1;         { Soletrao braille Unificado }
    SOLETRAAMERICANBRAILLE = 2; { Soletrando american Braille code }
    SOLETRANORMAL = 3;          { Soletrao padro }

var
    modoSoletrar: integer = 0;  { 0 - normal; 1 - Braille unificado; 2 - American Braille Code. }

implementation

{--------------------------------------------------------}

const
    { Patrick. Tabela transcrita do BRAILEX.AMB }
    tabCaracAMBCode : array[#$20..#$7F] of string[6]=(
        { } '0',
        {!} '2346',
        {"} '5',
        {#} '3456',
        {cifro} '1246',
        {%} '146',
        {&} '12346',
        {'} '3',
        {(} '12356',
        {)} '23456',
        {*} '16',
        {+} '346',
        {,} '6',
        {-} '36',
        {.} '46',
        {/} '34',
        {0} '356',
        {1} '2',
        {2} '23',
        {3} '25',
        {4} '256',
        {5} '26',
        {6} '235',
        {7} '2346',
        {8} '236',
        {9} '35',
        {:} '156',
        {;} '56',
        {<} '126',
        {=} '123456',
        {>} '345',
        {?} '1456',
        {@} '4',
        {A} '1',
        {B} '12',
        {C} '14',
        {D} '145',
        {E} '15',
        {F} '124',
        {G} '1245',
        {H} '125',
        {I} '24',
        {J} '245',
        {K} '13',
        {L} '123',
        {M} '134',
        {N} '1345',
        {O} '135',
        {P} '1234',
        {Q} '12345',
        {R} '1235',
        {S} '234',
        {T} '2345',
        {U} '136',
        {V} '1236',
        {W} '2456',
        {X} '1346',
        {Y} '13456',
        {Z} '1356',
        {[} '246',
        {\} '1256',
        {]} '12456',
        {^} '45',
        {_} '456',
        {`} '4',
        {a} '1',
        {b} '12',
        {c} '14',
        {d} '145',
        {e} '15',
        {f} '124',
        {g} '1245',
        {h} '125',
        {i} '24',
        {j} '245',
        {k} '13',
        {l} '123',
        {m} '134',
        {n} '1345',
        {o} '135',
        {p} '1234',
        {q} '12345',
        {r} '1235',
        {s} '234',
        {t} '2345',
        {u} '136',
        {v} '1236',
        {w} '2456',
        {x} '1346',
        {y} '113456',
        {z} '1356',
        {abre chave} '246',
        {|} '1256',
        {fecha chave} '12456',
        {~} '45',
        {} '123456'
    );

    { Patrick. Tabela Braille Unificado }
    tabCaracBrUnif : array[#$20..#$FF] of string[10] = (
        { }  '0',
        {!} '235',
        {"} '236',
        {#} '3456',
      {cif} '56',
        {%} '456 356',
        {&} '12346',
        {'} '3',
        {(} '126 3',
        {)} '6 345',
        {*} '35',
        {+} '235',
        {,} '2',
        {-} '36',
        {.} '3',
        {/} '6 5',
        {0} '3456 245',
        {1} '3456 1',
        {2} '3456 12',
        {3} '3456 14',
        {4} '3456 145',
        {5} '3456 15',
        {6} '3456 124',
        {7} '3456 1245',
        {8} '3456 125',
        {9} '3456 24',
        {:} '25',
        {;} '23',
        {<} '246',
        {=} '2356',
        {>} '135',
        {?} '26',
        {@} '156',
        {A} '1',
        {B} '12',
        {C} '14',
        {D} '145',
        {E} '15',
        {F} '124',
        {G} '1245',
        {H} '125',
        {I} '24',
        {J} '245',
        {K} '13',
        {L} '123',
        {M} '134',
        {N} '1345',
        {O} '135',
        {P} '1234',
        {Q} '12345',
        {R} '1235',
        {S} '234',
        {T} '2345',
        {U} '136',
        {V} '1236',
        {W} '2456',
        {X} '1346',
        {Y} '13456',
        {Z} '1356',
        {[} '12356 3',
        {\} '5 3',
        {]} '6 23456',
        {^} '4',
        {_} '456',
        {`} '6',
        {a} '1',
        {b} '12',
        {c} '14',
        {d} '145',
        {e} '15',
        {f} '124',
        {g} '1245',
        {h} '125',
        {i} '24',
        {j} '245',
        {k} '13',
        {l} '123',
        {m} '134',
        {n} '1345',
        {o} '135',
        {p} '1234',
        {q} '12345',
        {r} '1235',
        {s} '234',
        {t} '2345',
        {u} '136',
        {v} '1236',
        {w} '2456',
        {x} '1346',
        {y} '13456',
        {z} '1356',
      {chv} '5 123',
        {|} '456',
      {fcv} '456 2',
        {~} '5',
        {} '123456',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {no usamos} '0',
        {pargrafo} '234 234',
        {trema} '0',
        {copyright} '0',
        {ordinal feminino} '1',
        {abre-aspas literrio} '6 236',
        {negao lgica} '0',
        {soft hyphen} '0',
        {registrado} '0',
        {overscore} '0',
        {grau} '356',
        {mais ou menos} '0',
        {superscrito 2} '16 # 12',
        {superscrito 3} '16 #14',
        {acento agudo} '35',
        {micro} '134', //M maiscula
        {pargrafo} '5 346', //Outro cdigo para pargrafo
        {middle dot} '0',
        {cedilha} '0',
        {superscrito 1} '16 #1',
        {ordinal masculino} '135',
        {fecha-aspas  literrio} '6 236',
        {1 quarto} '# 2 145',
        {1 meio} '# 2 12',
        {3 quartos} '# 25 145',
        {interrogao invertida} '26', //Igual a interrogao convencional
        {} '1246',
        {} '12356',
        {} '16',
        {} '345',
        {} '345',
        {} '16',
        {no usamos} '0',
        {} '12346',
        {} '2346',
        {} '123456',
        {} '126',
        {} '0',
        {} '146',
        {} '34',
        {} '0',
        {} '0',
        {no usamos} '0',
        { no usamos} '12456',
        {} '2456',
        {} '346',
        {} '1456',
        {} '246',
        {} '246',
        { multiplicao} '236',
        { vazio} '0',
        {} '0',
        {} '23456',
        {} '0',
        {} '1246',
        {} '0',
        { no usamos} '0',
        { no usamos} '0',
        {} '1246',
        {} '12356',
        {} '16',
        {} '345',
        {} '345',
        {} '16',
        { no usamos} '0',
        {} '12346',
        {} '2346',
        {} '123456',
        {} '126',
        {} '0',
        {} '146',
        {} '34',
        {} '0',
        { no usamos} '0',
        { no usamos} '0',
        { no usamos} '12456',
        {} '2456',
        {} '346',
        {} '1456',
        {} '246',
        {} '246',
        { diviso} '256',
        { no usamos} '0',
        {} '0',
        {} '23456',
        {} '0',
        {} '1256',
        { no usamos} '0',
        { no usamos} '0',
        { no usamos} '0'
    );

var
    soletrarMaiusculaBraille: boolean;
    pausaEntreCelBraille: integer;

{--------------------------------------------------------}

procedure soletraPontosBraille(c: char; modo: integer);

    procedure soletraPontosBraille (s: string);
    begin
        while pos(' ', s) <> 0 do
            begin
                sintSoletra (copy(s, 1, pos(' ', s)-1));
                delete (s, 1, pos(' ', s));
                if length(s) > 0 then delay (pausaEntreCelBraille);
            end;
        sintSoletra (s);
    end;

    function estaEmMaiusculo (c: char): boolean;
    begin
        result := (c = upcase(c)) and
                (not(c in ['0' .. '9', ''.. '', '' .. '', '' .. '', '' .. '', '',
                        '.', ',', ':', ';', '?', '!', '/','\', '|', '''', '"','@', '#', '$',
                        '%', '&', '*', '(', ')', '_', '-', '+', '=', '[', ']', '{', '}']));
    end;

begin
    if c = ' ' then sintCarac(' ')
    else
        begin
            if soletrarMaiusculaBraille and  estaEmMaiusculo(c) then
                begin
                    sintSoletra ('46');
                    delay (pausaEntreCelBraille);
                end;

            if modo = 1 then
                soletraPontosBraille (tabCaracBrUnif[c])
            else { modo = 2}
                begin
                    if (c < #$20) or (c > #$7F) then
                        begin
                        sintClek;
                        sintClek;
                    end
                    else
                        soletraPontosBraille (tabCaracAMBCode[c]);
                end;
        end;
end;

{--------------------------------------------------------}

procedure trocaModoSoletrar;
begin
    modoSoletrar := (modoSoletrar + 1) mod 3;
    case modoSoletrar of
        0: fala ('EDSOLNOR');  {'Soletrao normal'}
        1: fala ('EDSOLBRL');  {'Soletrao braille'}
        2: fala ('EDSOLAMB');  {'Soletrao American Braille Code'}
    end;
end;

{--------------------------------------------------------}

var erro: integer;
begin
    soletrarMaiusculaBraille := upcase(sintAmbiente ('EDIVOX', 'SOLETRARMAIUSCULABRAILLE', 'NAO')[1]) = 'S';
    val (sintAmbiente ('EDIVOX', 'PAUSAENTRECELBRAILLE', '200'), pausaEntreCelBraille, erro);
    if erro <> 0 then pausaEntreCelBraille := 200;
end.
