{--------------------------------------------------------}
{
{    Jogavox - criador de jogos educacionais
{
{    Mdulo de baixar
{
{    Autores: Jos Antonio Borges
{             Lidiane Figueira Silva
{             Bernard Condorcet
{             Marcolino Nascimento
{
{    Em Junho/2015
{
{--------------------------------------------------------}

unit jobaixa;

interface

uses
    sysutils,
    dvwin,
    dvcrt,
    dvform,
    dvinet,
    synacode,
    classes,
    jomsg,
    jovars,
    dvexec,

    Windows;

procedure selCategoria;
procedure Cabecalho;
procedure baixaJogo(categoria:string);

implementation

{--------------------------------------------------------}
{                  Obtm o diretrio do Dosvox
{--------------------------------------------------------}

function obtemDirDosvox: String;
var
    dir: String;
begin
    dir := SintAmbiente('DOSVOX', 'PGMDOSVOX');
    if (dir = '') or (dir = '@') then
        dir := 'C:\Winvox';

    result := dir;
end;

{--------------------------------------------------------}
{               Faz o download de um arquivo
{--------------------------------------------------------}

function baixaArquivo (url, arquivoABaixar: string; aceitaErro: boolean): boolean;

const
    CRLF = #$0d + #$0a;

var
    statusUltIO: integer;

{--------------------------------------------------------}
{                  Traduz a url
{--------------------------------------------------------}

function traduzURL (url: string; out protocolo, nomeComput: string;
                                 out porta: integer;
                                 out recurso: string): boolean;
var
    i: integer;
    erro: integer;
    s: string;

begin
    traduzURL := false;

    url := trim(url);
    i := pos('://' , url);

    if i = 0 then
        protocolo := 'http'
    else
    begin
        protocolo := copy(url, 1, (i-1));
        url := copy(url, (i+3), 999);
    end;
        protocolo := upperCase(protocolo);

        i := pos('/', url);
        if i = 0 then
            i := pos('?', url);

        if i = 0 then
            begin
                recurso := '';
                nomeComput := url;
            end
        else
            begin
                nomeComput := copy(url, 1, (i-1));
                recurso := copy(url, i, 999);
                if copy(recurso, 1,1) = '?' then
                    recurso := '/' + recurso;
            end;

        i := pos(':', nomeComput);
        if i <> 0 then
            begin
                s := copy(nomeComput, (i+1), 999);
                nomeComput := copy(nomeComput, 1, i-1);
                val (s, porta, erro);
                if erro <> 0 then
                    exit;
            end
        else
            if protocolo = 'HTTPS' then
                porta := 443
            else
                porta := 80;

        if recurso = '' then
            recurso := '/';

        i := pos('?', recurso);
        if i <> 0 then
            recurso := copy(recurso, 1,i) + EncodeURL(copy(recurso, i+1,999));

        traduzURL := true;
end;


{--------------------------------------------------------}
{                  Pega o cabealho
{--------------------------------------------------------}

function pegaHeader (protocolo, nomeComput: string; porta: integer; recurso: string;
                         out codRetorno: integer;
                         out novaUrl: string;
                         out pbuf: PbufRede;
                         out soquete: integer): boolean;

var s: string;
    i: integer;
    header: TStringList;
    aEnviar: string;

begin
    pegaHeader := false;
    codRetorno := 500;
    novaUrl := '';

    if ansiUpperCase(protocolo) = 'HTTPS' then
        soquete := abreConexaoSSL (nomeComput, porta)
    else
        soquete := abreConexao (nomeComput, porta);

    if soquete < 0 then
       begin
           exit;
       end;

    aEnviar :=
        'GET ' + EncodeURL(recurso) + ' HTTP/1.0' + CRLF +
        'UA-CPU: x86' + CRLF +
        'Connection: Close' + CRLF +
        'Accept-Language: pt-br' + CRLF +
        'User-Agent: Webvox 2.4' + CRLF +
        'Host: ' + nomeComput + CRLF +
        CRLF;

    statusUltIO := ord (writeRede(soquete, aEnviar));

    pBuf := inicBufRede (soquete);

    header := TStringList.Create;
    repeat
        statusUltIO := ord (not readlnBufRede (pbuf, s, 30));
        header.add(s);
    until (statusUltIO <> 0) or (s = '');

    if copy(header[0], 1,4) <> 'HTTP' then   // erro no servidor
        begin
                sintWriteln ('Erro no servidor');
                header.Free;
                fechaConexao(soquete);
                fimBufRede(pbuf);
                pbuf := NIL;
                exit;
            end;

        s := header[0];
        i := pos(' ', s);
        codRetorno := StrToInt(copy(s, i+1, 3));

        if (codRetorno div 100) = 3  then  // relocators
            begin
                // pega o location
                for i := 0 to (header.Count-1) do
                    begin
                        if pos('LOCATION:', upperCase(header[i])) = 1 then
                            novaUrl := trim(copy(header[i], 10 , 999));
                    end;
                fimBufRede (pbuf);
                fechaConexao(soquete);
            end;

        header.Free;
        pegaHeader := true;
    end;

{--------------------------------------------------------}
{                  abre a url
{--------------------------------------------------------}

function abreUrl(url: string; out pBuf: pbufrede; out soquete: integer): boolean;
var
    protocolo, nomeComput, recurso: string;
    porta: integer;
    novaUrl: string;
    codRetorno: integer;

begin
    abreUrl := false;
    novaUrl := url;

    codRetorno := 300;
    while (codRetorno div 100) = 3  do
        begin
            if not traduzURL (novaUrl, protocolo, nomeComput, porta, recurso) then
                exit;

            if not pegaHeader (protocolo, nomeComput, porta, recurso,
                                   codRetorno, novaUrl, pbuf, soquete) then
                    exit;
       end;

    abreUrl := true;
end;

{--------------------------------------------------------}
{          Copia contedo da url para um arquivo
{--------------------------------------------------------}

function copiaURLparaArquivo (pbuf: PbufRede; soquete: integer;
                              nomeArqBaixar: string): boolean;
const
    TAMBUF = 8192;
var
    arq: file;
    lidoOk, clecando: boolean;
    buf: packed array [0..TAMBUF-1] of char;
    ncbuf: integer;
    c: char;
    escritos: integer;
begin
     copiaURLparaArquivo := false;
     statusUltIO := 0;
     ncbuf := 0;
     clecando := True;

     assign (arq, nomeArqBaixar);
     {$I-}  rewrite (arq, 1);  {$I+}
     if ioresult <> 0 then
         begin
             statusUltIO := 1;
             mensagem('JOERROES', 1); // Erro de escrita
             exit;
         end;
    repeat
        lidoOk := leCaracBufRede(pbuf, c);
        if lidoOk then
            begin
                buf[ncbuf] := c;
                ncbuf := ncbuf + 1;
            end;

        if ncbuf >= TAMBUF then
            begin
                if clecando then
                    sintClek;
                if keypressed then
                    if readkey = ' ' then clecando := not clecando;
                escritos := 0;
                blockWrite (arq, buf, ncbuf, escritos);
                if escritos <> ncbuf then
                    begin
                        mensagem('JOERROES', 1); // Erro de escrita
                        statusUltIO := 1;
                        lidoOk := false;
                    end;
                ncbuf := 0;
            end;
    until not lidoOk;

    limpaBufTec;
    if ncbuf <> 0 then
        begin
            blockWrite (arq, buf, ncbuf, escritos);
            if escritos <> ncbuf then
                begin
                    mensagem('JOERROES', 1); // Erro de escrita    
                    statusUltIO := 1;
                end;
        end;

    closeFile (arq);
    copiaURLparaArquivo := true;
end;

{--------------------------------------------------------}

var
    pbuf: PbufRede;
    soquete: integer;

label erro;
begin
    result := false;

    if abreUrl(url, pbuf, soquete) then
        begin
          if copiaURLparaArquivo (pbuf, soquete, arquivoABaixar) or aceitaErro then
                result := true;
          fimBufRede(pbuf);
          fechaConexao(soquete);
        end;
end;

{--------------------------------------------------------}
{                  Limpa tags html do arquivo
{--------------------------------------------------------}

function substitui(linha: string): string;
begin
    linha := StringReplace(linha, '</td>', ^m^j , [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '</tr>', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '<tr><td valign="top"><img src=', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '<tr>', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '&nbsp;', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '<td>', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '<td align="right">', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '<tr><td valign="top">', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '<a href=', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '<img src=', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, 'alt="[   ]">', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '</a>', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '<th colspan="5"><hr></th>', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '</body></html>', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '</table>', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '<a href=', '', [rfReplaceAll, rfIgnoreCase]);
    linha := StringReplace(linha, '">', '"'+^m^j, [rfReplaceAll, rfIgnoreCase]);
    result := linha;
end;

{--------------------------------------------------------}
{            Descobre diretrio de trabalho
{--------------------------------------------------------}

function GetTempDir: string;
var
  Buffer: array[0..512] of Char;

begin
    GetTempPath(512,Buffer);
    Result := StrPas(Buffer);
end;

{--------------------------------------------------------}
{                  Abre o arquivo txt
{--------------------------------------------------------}

function abreArquivo(endereco:string):TStringList;
var
    linha: string;
    arquivo: text;
    lista: TStringList;
    i: integer;

begin
    AssignFile(arquivo,endereco);
    {$I-}
    Reset(arquivo);
    {$I+}

    if ioresult <> 0 then
        begin
            mensagem ('JOBAARQE', 1);    //'Arquivo jogos.$$$ no foi achado.'
            halt;
        end;

    lista := TStringList.Create;
    while (not eof(arquivo)) do
         begin
           readln(arquivo,linha);
           linha := substitui (linha);
           lista.add (linha);
         end;

    for i := 1 to 11 do  // apaga cabealho
        lista.delete(0);

    result:=lista;
    closefile(arquivo);
end;

{--------------------------------------------------------}
{                  Cria listaJogos.txt
{--------------------------------------------------------}

procedure criaListaJogos(lista:TStringList);

begin
    lista.SaveToFile(dirBaseJogos+'\listajogos.$$$');
    lista.LoadFromFile(dirBaseJogos+'\listajogos.$$$');
end;

{--------------------------------------------------------}
{     Remove sublinhados e transforma de UTF para Ansi
{--------------------------------------------------------}

function simplif (s: string): string;
begin
     result := Utf8ToAnsi(StringReplace(s, '_', ' ', [rfReplaceAll, rfIgnoreCase]));
end;

{--------------------------------------------------------}
{                 Cria o menu
{--------------------------------------------------------}

procedure criaMenu(lista:TStringList; categoria: String);
var
    tam,i: integer;

begin
    writeln(categoria);
    writeln;
    mensagem ('JOESCJOG', 1);  {'Escolha um dos seguintes jogos com as setas'}
    garanteEspacoTela (7);

    tam:=60;
    popupMenuCria(wherex, wherey, tam, 26-wherey, RED);

    for i := 0 to (lista.Count div 7)-1 do
        popupMenuAdiciona('', simplif(lista[i*7+2]));
end;

{--------------------------------------------------------}
{            Faz download da pagina de jogos
{--------------------------------------------------------}

procedure fazDownloadHtml(categoria: string);
begin
    abreWinSock;

    if not baixaArquivo (URL_JOGOS + categoria,
                dirBaseJogos+'\jogos.$$$', false) then
        mensagem('JOBAERRO', 1); //'Erro ao tentar baixar o Jogo.'
    fechaWinSock;
end;

{--------------------------------------------------------}
{                 Descomprimir o arquivo zip
{--------------------------------------------------------}

procedure descompacta(arquivo:string);
var
    extrator: String;
begin
    extrator := '"' + obtemDirDosvox + '\unzip.exe" -o';
    executaProgEX (extrator,'.',arquivo, SW_SHOWMINIMIZED);
    delay(1000);
    esperaProgVoltar;
    DeleteFile(Pchar(arquivo));
end;

{--------------------------------------------------------}
{                 Escolhe categoria do jogo
{--------------------------------------------------------}

procedure selCategoria;
var
    c: char;
    n: integer;
    opcao:string;
begin
    Cabecalho;
    gotoxy (1, 4);

    c := pergunta ('JOBACATE', 0, BLUE);  {'Opo: Olimpo, Gaia, Caos? '}

    if c = ESC then
        begin
            mensagem('JODESIST',3);  {'Desistiu'}
            exit;
        end;
    if c = #0 then
        begin
            popupMenuCria (wherex, wherey, 12, 3, MAGENTA);
            MenuAdiciona('JOOLIMPO'); // 'olimpo'
            MenuAdiciona('JOGAIA');   // 'gaia'
            MenuAdiciona('JOCAOS');   // 'caos'
            limpaBufTec;

            n := popupMenuSeleciona;

            case n of
                1:  opcao := pegaTextoMensagem('JOOLIMPO'); // 'olimpo'
                2:  opcao := pegaTextoMensagem('JOGAIA');   // 'gaia'
                3:  opcao := pegaTextoMensagem('JOCAOS');   // 'caos'
            else
                mensagem('JODESIST',3);  {'Desistiu'}
                exit;
            end;
        end

      else
        begin
            case upcase(c) of
                'O':  opcao := pegaTextoMensagem('JOOLIMPO');   // 'olimpo'
                'G':  opcao := pegaTextoMensagem('JOGAIA');     // 'gaia'
                'C':  opcao := pegaTextoMensagem('JOCAOS');     // 'caos'
            else
                mensagem('JODESIST',3);  {'Desistiu'}
                exit;
            end;
        end;
     baixaJogo(opcao);
end;

{--------------------------------------------------------}
{                 Cabecalho do programa
{--------------------------------------------------------}

procedure Cabecalho;
begin
    clrscr;
    setWindowTitle('Jogavox');
    textBackground (BLUE);
    limpaBufTec;

    write(pegaTextoMensagem('JOINIC'));     {'Jogavox - editor de jogos educacionais'}
    write (' - ');
    write(pegaTextoMensagem('JOVERSAO'));   {'Verso '}
    writeln (versao);
    textBackground (BLACK);
    writeln;
end;

{--------------------------------------------------------}
{                 Faz o download do jogo
{--------------------------------------------------------}

procedure fazDownload(lista:TStringList; opcao:integer; categoria: string);
var
    url, arquivoAGravar: string;

begin
    abreWinSock;
    mensagem ('JOBABXND', 0);   //'Baixando o jogo: '
    sintWriteln(simplif(lista[(opcao-1)*7+2])) ;

    url:=URL_JOGOS+ categoria+ '/' + lista[(opcao-1)*7+2];
    arquivoAGravar:=dirBaseJogos+'\'+lista[(opcao-1)*7+2];

    if not baixaArquivo (url, arquivoAGravar, false) then
        mensagem ('JOBAERRO', 1) // 'Erro ao tentar baixar o Jogo. Por favor, tente novamente.'
    else
        begin
            mensagem ('JOEXTARQ', 1); //'Extraindo o arquivo'
            descompacta(lista[(opcao-1)*7+2]);

            mensagem ('JOBA_BXD', 1); //'O seu jogo foi baixado com sucesso.'
            delay(1000);
        end;

    fechaWinSock;
end;

{--------------------------------------------------------}
{                 Controle geral do download
{--------------------------------------------------------}

procedure baixaJogo(categoria:string);
var
  lista:TStringList;
  opcao:integer;

begin
    fazDownloadHtml(categoria);
    lista := abreArquivo(dirBaseJogos+'\jogos.$$$');
    if (lista.Count < 2) or (lista[1] = '') then
        begin
            mensagem ('JOBAARQE', 2);   // 'Arquivo de jogos no foi achado.'
            exit;
        end;

    criaListaJogos(lista);
    criaMenu(lista, categoria);

    opcao := popupMenuSeleciona;         // futuramente permitir um loop de seleo.

    DeleteFile(Pchar(dirBaseJogos+'\jogos.$$$'));
    DeleteFile(Pchar(dirBaseJogos+'\listajogos.$$$'));

    if opcao >= 1 then
        fazDownload(lista,opcao,categoria)
    else
        mensagem('JODESIST',3);  {'Desistiu'}
end;

end.
