{--------------------------------------------------------}
{                                                        }
{    Radio50 - Executor interativo de streams de udio   }
{                                                        }
{    Rotinas de ajuda
{                                                        }
{    Autor:  Neno Henrique da Cunha Albernaz
{                                                        }
{    Em Dezembro/2021                                     }
{                                                        }
{--------------------------------------------------------}

unit rdAjuda;

interface
uses
    dvcrt,
    dvwin,
    Windows,
    dvForm,
    dvAmplia,
    sysUtils,
    rdmsg;

procedure ajudaOpcaoPrincipal;
function selSetasOpcaoPrincipal: char;
procedure ajudaOpcaoPreferidas;
function selSetasOpcaoPreferidas: char;
procedure ajudaFolheiaRadios;
function  selSetasFolheiaRadios (Var c2: char; var apertouShift: boolean): char;
procedure ajudaTocaRadioBass;
procedure ajudaTocaRadioExterna;

implementation

{--------------------------------------------------------}
{                        ajuda do menu principal do programa
{--------------------------------------------------------}

procedure ajudaOpcaoPrincipal;
begin
    writeln;
    mensagem ('RDOPCAO', 1);  {'As opes so:'}
    mensagem ('RDOPC_P', 1);  {'       P - radios preferidas'}
    mensagem ('RDOPC_F', 1);  {'       F - folheia as rdios'}
    mensagem ('RDOPC_B', 1);  {'       B - buscar uma rdio pelo nome'}
    mensagem ('RDOPC_A', 1);  {'       A - atualizar a lista de rdios por arquivo ATU'}
    mensagem ('RDOPC_T', 1);  {'       T - testar um endereo de rdio'}
//    mensagem ('RDOPC_E', 1);  {'       E - editar uma categoria'}
//    mensagem ('RDOPC_I', 1);  {'       I - incluir item em uma categoria'}
//    mensagem ('RDOPC_R', 1);  {'       R - remover item de uma categoria'}
//    mensagem ('RDOPC_C', 1);  {'       C - criar nova categoria'}
//    mensagem ('RDOPC_D', 1);  {'       D - destruir uma categoria'}
    mensagem ('RD_ESC' , 1);  {'     ESC - terminar'}
    mensagem ('RDSETOPC', 0);         {'Use as setas para conhecer outras opes'}

    while keypressed do readkey;
    sintBip;
end;

{--------------------------------------------------------}
{            seleciona a opo do menu principal do programa com as setas
{--------------------------------------------------------}

function selSetasOpcaoPrincipal: char;
var n: integer;
const
    tabLetrasOpcao: string = 'PFBATEIRCD' + ^P + ^B + ESC;

begin
    garanteEspacoTela (9);
    popupMenuCria (wherex, wherey, 50, length(tabLetrasOpcao), MAGENTA);
    menuAdiciona ('RDOPC_P');         {'       P - radios preferidas'}
    menuAdiciona ('RDOPC_F');         {'       F - folheia as rdios'}
    menuAdiciona ('RDOPC_B');         {'       B - buscar uma rdio pelo nome'}
    menuAdiciona ('RDOPC_A');         {'       A - atualizar a lista de rdios por arquivo ATU'}
    menuAdiciona ('RDOPC_T');         {'       T - testar um endereo de rdio'}
    menuAdiciona ('RDOPC_E');         {'       E - editar uma categoria'}
    menuAdiciona ('RDOPC_I');         {'       I - incluir item em uma categoria'}
    menuAdiciona ('RDOPC_R');         {'       R - remover item de uma categoria'}
    menuAdiciona ('RDOPC_C');         {'       C - criar nova categoria'}
    menuAdiciona ('RDOPC_D');         {'       D - destruir uma categoria'}
    menuAdiciona ('RDOPC_CTRLP');     {'Ctrl + P - folhear preferidas sem sair'}
    menuAdiciona ('RDOPC_CTRLB');     {'Ctrl + B - buscar pelo nome sem sair'}
    menuAdiciona ('RD_ESC' );         {'     ESC - terminar'}

    n := popupMenuSeleciona;

    if n > 0 then
        begin
            result := tabLetrasOpcao[n];
            writeln (tabLetrasOpcao[n]);
        end
    else
        result := #0;
end;

{--------------------------------------------------------}
{                        ajuda das preferidas
{--------------------------------------------------------}

procedure ajudaOpcaoPreferidas;
begin
    writeln;
    mensagem ('RDOPCAO', 1);  {'As opes so:'}
    mensagem ('RDOPP_F', 1);  {'       F - folheia as preferidas'}
    mensagem ('RDOPP_P', 1);  {'       P - escolhe pelo nmero da preferida'}
    mensagem ('RDOPP_U', 1);  {'       U - ltima rdio escutada'}
    mensagem ('RD_ESC' , 1);  {'     ESC - terminar'}

    while keypressed do readkey;
    sintBip;
end;

{--------------------------------------------------------}
{            seleciona a opo das preferidas com as setas
{--------------------------------------------------------}

function selSetasOpcaoPreferidas: char;
var n: integer;
const
    tabLetrasOpcao: string = 'FPU' + ESC;

begin
    garanteEspacoTela (5);
    popupMenuCria (wherex, wherey, 50, length(tabLetrasOpcao), MAGENTA);
    menuAdiciona ('RDOPP_F');  {'       F - folheia as preferidas'}
    menuAdiciona ('RDOPP_P');  {'       P - escolhe pelo nmero da preferida'}
    menuAdiciona ('RDOPP_U');  {'       U - ltima rdio escutada'}
    menuAdiciona ('RD_ESC' );  {'     ESC - terminar'}
    n := popupMenuSeleciona;
    if n > 0 then
        begin
            result := tabLetrasOpcao[n];
            writeln (tabLetrasOpcao[n]);
        end
    else
        result := ESC;
end;

{--------------------------------------------------------}
{       ajuda no folheamento das rdios
{--------------------------------------------------------}

procedure ajudaFolheiaRadios;
begin
    cabecalho (false);
    textBackground(BLUE);
    mensagem ('RDAJFORA', 2);  {'Folheie as radios com as setas, depois tecle:'}
    textBackground(BLACK);

    mensagem ('RDAJFO_ENTER', 1);      {'       Enter    - tocar rdio'}
    mensagem ('RDAJFO_CTRLP', 1);      {'Ctrl + P        - adicionar as preferidas'}
    mensagem ('RDAJFO_CTRLE', 1);      {'Ctrl + E        - editar rdio'}
    mensagem ('RDAJFO_CTRLR', 1);      {'Ctrl + R        - remover rdio'}
    mensagem ('RDAJFO_CTRLC', 1);      {'Ctrl + C        - copiar para rea de transferncia'}
    mensagem ('RDAJFO_ESC', 2);        {'ESC terminar folheamento'}
    mensagem ('RDAJFO_F9', 0);         {'Tecle F9 para conhecer outras opes'}

    textBackground(BLACK);  clreol;
    writeln;
    if keypressed and (readkey <> ESC) then
        begin
            limpaBufTec;
            readkey;
        end;
    limpaBufTec;
end;

{--------------------------------------------------------}
{       Selecionar com as setas as opes no folheamento das rdios
{--------------------------------------------------------}

function selSetasFolheiaRadios (Var c2: char; var apertouShift: boolean): char;
var
    n: integer;
    s: string;
const
    tabOpc: string = ENTER + ^P + ^E + ^R + ^Q + ^Q + ^C + ^T + '3' + ESQ + CTLESQ + DIR + CTLDIR + F5 + CTLF5 + ESC;

begin
    C2 := #0;

    popupMenuCria (35, wherey, 50, length(tabOpc), RED);
    MenuAdiciona ('RDAJFO_ENTER');     {'               Enter    - tocar rdio'}
    MenuAdiciona ('RDAJFO_CTRLP');     {'        Ctrl + P        - adicionar as preferidas'}
    MenuAdiciona ('RDAJFO_CTRLE');     {'        Ctrl + E        - editar rdio'}
    MenuAdiciona ('RDAJFO_CTRLR');     {'        Ctrl + R        - remover rdio'}
    MenuAdiciona ('RDAJFO_CTRLQ');     {'        Ctrl + Q        - posio atual e total de rdios'}
    MenuAdiciona ('RDAJFO_CTRLSFTQ');  {'        Ctrl + Shift + Q        - selecionados e total de rdios'}
    MenuAdiciona ('RDAJFO_CTRLC');     {'        Ctrl + C        - copiar para rea de transferncia'}
    MenuAdiciona ('RDAJFO_CTRLT');     {'       Ctrl + T        - buscar rdio que usa tocador externo'}
    MenuAdiciona ('RDAJFO_3');         {'               3        - gerar arquivos m3u'}
    MenuAdiciona ('RDAJFO_ESQ');       {'               Esquerda - fala categoria'}
    MenuAdiciona ('RDAJFO_CTRLESQ');   {'        Ctrl + esquerda - soletra categoria'}
    MenuAdiciona ('RDAJFO_DIR');       {'               direita  - fala site'}
    MenuAdiciona ('RDAJFO_CTRLDIR');   {'        Ctrl + direita  - soletra site'}
    MenuAdiciona ('RDAJFO_F5');        {'               F5       - busca'}
    MenuAdiciona ('RDAJFO_CTRLF5');    {'        Ctrl + F5       - busca novamente'}
    MenuAdiciona ('RDAJFO_ESC');       {'               ESC      - terminar folheamento'}

    n := popupMenuSeleciona;

    if (n >= 1) and(n <= 9) or (n = length(tabOpc)) then
        begin
            s :=  maiuscansi(opcoesItemSelecionado );
            apertouShift := pos ('SHIFT +', s) > 0;
            result := tabOpc[n];
        end
    else
    if (n > 8) and (n < length(tabOpc)) then
        begin
            c2 := tabOpc[n];
            result := #0;
        end
    else
        result := #0;
end;

{--------------------------------------------------------}
{       ajuda da interao do tocaRadioBass
{--------------------------------------------------------}

procedure ajudaTocaRadioBass;
begin
    writeln;
    mensagem ('RDOPCSAO', 1);    {'As opes so:'}
    mensagem ('RDESPACO', 1);    {'espao - toca ou para'}
    mensagem ('RDOPGRAVAR', 1);    {'G - gravar rdio'}
    if upcase(sintAmbiente ('RADIO50', 'JANELAGRAVACAOAPARENTE', 'NAO')[1]) = 'N' then
        mensagem ('RDOPFIGRAVAR', 1);    {'F - finalizar gravao'}
    mensagem ('RDOPNOMRADI', 1);    {'R - nome da rdio'}
    mensagem ('RDOPNOME', 1);    {'N - exibe nome reduzido'}
    mensagem ('RDOPVOL' , 1);    {'V - muda volume'}
    mensagem ('RDOPPARM', 1);    {'P - mostra parmetros de transmisso'}
    mensagem ('RDOPENDR', 1);    {'E - endereo de transmisso'}
    mensagem ('RDOPPROX', 1);    {'PAGE UP - prxima programao'}
    mensagem ('RDOPANT',  1);    {'PAGE DOWN - programao anterior'}
    mensagem ('RDOPESC',  1);    {'ESC - termina'}
    writeln;
    TextBackground (BLUE);
    mensagem ('RDQUALOP',  0);   {'Qual sua opo? '}
    TextBackground (BLACK);
end;

{--------------------------------------------------------}
{       ajuda da interao do tocaRadioExterna
{--------------------------------------------------------}

procedure ajudaTocaRadioExterna;
begin
    writeln;
    TextBackground (BLUE);
    mensagem ('RDOPCSAO', 0);    {'As opes so:'}
    TextBackground (BLACK);
    writeln;
//    mensagem ('RDESPACO', 1);    {'espao - toca ou para'}
    mensagem ('RDOPGRAVAR', 1);    {'G - gravar rdio'}
    if upcase(sintAmbiente ('RADIO50', 'JANELAGRAVACAOAPARENTE', 'NAO')[1]) = 'N' then
        mensagem ('RDOPFIGRAVAR', 1);    {'F - finalizar gravao'}
    mensagem ('RDOPNOMRADI', 1);    {'R - nome da rdio'}
//    mensagem ('RDOPNOME', 1);    {'N - exibe nome reduzido'}
//No funciona    mensagem ('RDOPVOL' , 1);    {'V - muda volume'}
    mensagem ('RDOPINFOR', 1);    {'I - informao'}
    mensagem ('RDOPENDR', 1);    {'E - endereo de transmisso'}
    mensagem ('RDOPESC',  1);    {'ESC - termina'}
    writeln;
end;

{--------------------------------------------------------}

begin
end.
