{--------------------------------------------------------}
{                                                        }
{    Programa de acesso a redes sociais                  }
{                                                        }
{    Tratamento de Oauth 2.0                             }
{                                                        }
{    Autor: Jos Antonio Borges                          }
{                                                        }
{    Em junho/2011                                       }
{                                                        }
{--------------------------------------------------------}

unit twOAuth;

interface
uses
     windows, winsock, sysutils, classes,
     dvcrt, dvwin, dvInet, dvSsl, dvexec, shellapi,
     twmsg, twgzip, sha1;

procedure setOauthConsumer (consumer, secret, site, referUrl: string);
function OauthAuthorization (token_type, access_token,
                             oauth_signature, oauth_nonce, oauth_timestamp,
                             pin: string): string;
function OauthSignature (Method, Url, TStamp, Nonce,
                            ConsumerKey, ConsumerSecret,
                            OAuthToken, OAuthTokenSecret,
                            CallBack, Pin,
                            postData: string): string;
function getOAuthToken (token_type: string;
                        access_token, access_secret, pin: string;
                        var new_access_token, new_access_secret: string): boolean;
function GetDefaultBrowser: string;
procedure browseOAuthAuthenticate (temp_access_token, screen_name: string);
function getOAuthAutentication (screen_name: string;
         var oauth_token, oauth_secret: string): boolean;
function getOauthTwitterRest (Url: string; var status: integer): string;
function postOauthTwitterRest (Url, PostData: string; var status: integer): string;

function UrlEncode2(s: String): String;
function httpTransport (httpMsg: string; var status: integer): string;

var
    oauth_token, oauth_secret: string;
    oauth_site: string;
    refUrl: string;

implementation
uses twapi;

const
    default_oauth_site = 'api.twitter.com';           // defaults for twitvox
    default_refURL = 'http://intervox.nce.ufrj.br/dosvox';
    default_consumer_key = 'rda2P9PLNyeiTPSY6PUag';
    default_consumer_secret = 'CjBXsjhRP7RyGVwtS92MWgAwHJUkvRBa0X2DzsfMhU';

var
    consumer_key, consumer_secret: string;

{--------------------------------------------------------}
{                     URL encoding                       }
{--------------------------------------------------------}

function UrlEncode2(s: String): String;
var i: Integer;
begin
    Result := '';
    for i := 1 to length(s) do
        begin
            if s[i] in ['0'..'9', 'a'..'z', 'A'..'Z', '.', '-', '_', '~'] then
                Result := Result + s[i]
            else
                Result := Result + '%' + IntToHex(Ord(s[i]), 2);
        end;
end;

{--------------------------------------------------------}
{                   get time stamp                       }
{--------------------------------------------------------}

function GetUTCUnixTime: Int64;
var UTC: TSystemTime;
    t  : TDateTime;
begin
    GetSystemTime(UTC);
    t := SystemTimeToDateTime(UTC);
    Result := Round((t - 25569) * 86400);
end;

function GetTimeStamp: string;
begin
    result := IntToStr(GetUTCUnixTime);
end;

{--------------------------------------------------------}
{         generate an "unique" 20 digit integer          }
{--------------------------------------------------------}

function GetNonce: string;
var a: Integer;
    Nonce: string;
begin
    Nonce := '';
    for a := 1 to 20 do
        Nonce := Nonce + Chr(Random(26)+65);
    result := Nonce;
end;

{--------------------------------------------------------}
{               encode string in base64                  }
{--------------------------------------------------------}

function Base64Encode (bloco: string): string;
var
   s: string;
   lidos, guarda: integer;
   byteEnt: byte;
   noBloco, indBloco: integer;

const
    MIME64: array [0..63] of char =
       'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';

begin
   lidos := 0;
   s := '';
   noBloco := length (bloco);
   indBloco := 1;
   guarda := 0;

   while noBloco <> 0 do
       begin
           byteEnt := ord(bloco [indBloco]);
           indBloco := indBloco + 1;
           noBloco := noBloco - 1;

           case lidos of
               0: begin
                      s := s + MIME64[byteEnt shr 2];
                      guarda := (byteEnt and 3) shl 4;
                  end;
               1: begin
                      s := s + MIME64[((byteEnt shr 4) and $f) or guarda];
                      guarda := (byteEnt and $f) shl 2;
                  end;
               2: begin
                      s := s + MIME64[((byteEnt shr 6) and $3) or guarda];
                      guarda := byteEnt and $3f;
                      s := s + MIME64[guarda];
                  end;
           end;
           lidos := (lidos + 1) mod 3;
       end;

   if lidos <> 0 then
       s := s + MIME64[guarda];

    while (length (s) mod 4) <> 0 do
        s := s + '=';

    result := s;
end;

{--------------------------------------------------------}
{                 HTTP protocol processing               }
{--------------------------------------------------------}

function httpTransport (httpMsg: string; var status: integer): string;
var
    n, falta: integer;
    sockHTTP: integer;
    pbuf: PbufRede;
    s, header: string;
    p: pchar;
    aReceber: integer;
    i: integer;
    c: char;
    emGzip: boolean;
    sai: string;
label erroHTTP;
begin
    result := '';
    status := -1;

    sockHTTP := abreConexaoSSL (oauth_site, 443);
    if sockHTTP <= 0 then exit;

    falta := length(httpMsg);
    p := pchar(httpMsg);
    repeat
         n := sendBuf (sockHTTP, p, falta, 0);
         p := p + n;
         falta := falta - n;
    until (falta = 0) or (n <= 0);

    pbuf := inicBufRede (sockHTTP);
    if not readlnBufRede (pbuf, header, 20) then goto erroHttp;

    if copy (header, 1, 4) <> 'HTTP' then
        goto erroHttp;

    delete (header, 1, pos(' ', header));
    delete (header, pos(' ', header), 999);
    try
        status := strToInt (trim(header));
    except
        status := -1;
    end;

    if status = -1 then goto erroHTTP;

    aReceber := 0;
    emGzip := false;
    repeat
        if not readlnBufRede (pbuf, s, 10) then goto erroHttp;
        if upperCase (copy (s, 1, 15)) = 'CONTENT-LENGTH:' then
            aReceber := strToInt (trim(copy (s, 16, 999)))
        else
        if upperCase(copy (s, 1, 17)) = 'CONTENT-ENCODING:' then
            emGzip := trim (copy (s, 18, 999)) = 'gzip';
    until s = '';

    sai := '';
    if aReceber = 0 then
        aReceber := 999999;
    for i := 1 to aReceber do
        begin
            if leCaracBufRede (pbuf, c) then
                sai := sai + c;
        end;

    if emGzip then
        result := descompGzip (sai)
    else
        result := sai;

    fechaConexao(sockHTTP);
    exit;

erroHttp:
    fechaConexao(sockHTTP);
    mensagem ('TWERHTTP', 1);  {'Erro no protocolo HTTP'}
end;

{--------------------------------------------------------}
{              set oauth site and consumer               }
{--------------------------------------------------------}

procedure setOauthConsumer (consumer, secret, site, referUrl: string);
begin
    if site = ''     then oauth_site := default_oauth_site
                     else oauth_site := site;
    if consumer = '' then consumer_key := default_consumer_key
                     else consumer_key := consumer;
    if secret = ''   then consumer_secret := default_consumer_secret
                     else consumer_secret := secret;
    if referUrl = '' then refUrl := default_refURL
                     else refUrl := referURL;
end;

{--------------------------------------------------------}
{            generate authorization string               }
{--------------------------------------------------------}

function OauthAuthorization (token_type, access_token,
                             oauth_signature, oauth_nonce, oauth_timestamp,
                             pin: string): string;
var httpMsg: string;
begin
    httpMsg :=
        'Authorization: '+
            'OAuth ';
    if token_type = 'request_token' then httpMsg := httpMsg +
            'oauth_callback="oob",';
    httpMsg := httpMsg +
            'oauth_consumer_key="' + consumer_key + '"' +
            ',oauth_nonce="' + oauth_nonce + '"' +
            ',oauth_signature="' + UrlEncode2(oauth_signature) + '"' +
            ',oauth_signature_method="HMAC-SHA1"' +
            ',oauth_timestamp="' + oauth_timestamp + '"';
    if access_token <> '' then httpMsg := httpMsg +
            ',oauth_token="' + access_token + '"';
    if pin <> '' then httpMsg := httpMsg +
            ',oauth_verifier="' + pin + '"';
    httpMsg := httpMsg +
            ',oauth_version="1.0"';
    result := httpMsg;
end;

{--------------------------------------------------------}
{             For signature check, access                }
{             http://quonos.nl/oauthTester/              }
{--------------------------------------------------------}

function OauthSignature (Method, Url, TStamp, Nonce,
                            ConsumerKey, ConsumerSecret,
                            OAuthToken, OAuthTokenSecret,
                            CallBack, Pin,
                            postData: string): string;
var
    i, p: integer;
    //apiMethod,
    urlParams, httpReqString: AnsiString;
    SignBase, SignKey: AnsiString;
    parList: TStringList;

begin
    urlParams := '';
    p := pos ('?', Url);
    if p <> 0 then
        begin
            urlParams := copy (Url, p+1, 999);
            delete (Url, p, 999);
        end;

    parList := TStringList.Create;
    //if apiMethod <> '' then             // Twitter api no usa Method
    //    parList.add ('method=' + apiMethod);
    parList.add ('oauth_consumer_key=' + ConsumerKey);
    parList.add ('oauth_nonce=' + Nonce);
    parList.add ('oauth_signature_method=HMAC-SHA1');
    parList.add ('oauth_timestamp=' + TStamp);
    parList.add ('oauth_version=1.0');
    if OAuthToken <> '' then
        parList.add ('oauth_token='     + OAuthToken);
    if Callback <> '' then
        parList.add ('oauth_callback=' + Callback);
    if Pin <> '' then
        parList.add ('oauth_verifier='  + Pin);

    while urlParams <> '' do
        begin
            p := pos ('&', urlParams);
            if p = 0 then
                p := length (urlParams) + 1;
            parList.add (copy (urlParams, 1, p-1));
            delete (urlParams, 1, p);
        end;

    parList.sort;
    for i := 0 to parList.count-1 do
        httpReqString := httpReqString + '&' + parList[i];
    delete (httpReqString, 1, 1);
    parlist.free;

    SignBase := Method + '&'
              + URLEncode2(Url) + '&'
              + URLEncode2(httpReqString);
    if postData <> '' then
        SignBase := SignBase + URLEncode2('&' + postData);

    SignKey := URLEncode2(ConsumerSecret) + '&' + URLEncode2(OAuthTokenSecret);
    result := Base64Encode(HMAC_SHA1_EX(SignBase, SignKey));
end;

{--------------------------------------------------------}
{  Get Oauth Token (type=request_token or access_token)  }
{--------------------------------------------------------}

function getOAuthToken (token_type: string;
                        access_token, access_secret, pin: string;
                        var new_access_token, new_access_secret: string): boolean;
var
    httpMsg: ansiString;
    oauth_timestamp, oauth_nonce, oauth_signature: ansiString;
    status, n: integer;
    resp: string;
begin
    oauth_timestamp := GetTimeStamp;
    oauth_nonce := GetNonce;

    oauth_signature := OauthSignature ('POST',
        'https://' + oauth_site + '/oauth/request_token',
        oauth_timestamp, oauth_nonce,
        consumer_key, consumer_secret,
        access_token, access_secret,
        'oob', pin, '');

    httpMsg :=
        'POST ' + 'https://' + oauth_site + '/oauth/' + token_type + ' HTTP/1.0' + ^m^j +
        'User-Agent: TwitterVox' + ^m^j +
        'Host: api.twitter.com:443' + ^m^j +
        'Authorization: '+
            'OAuth ';
    if token_type = 'request_token' then httpMsg := httpMsg +
            'oauth_callback="oob",';
    httpMsg := httpMsg +
            'oauth_consumer_key="' + consumer_key + '"' +
            ',oauth_nonce="' + oauth_nonce + '"' +
            ',oauth_signature="' + UrlEncode2(oauth_signature) + '"' +
            ',oauth_signature_method="HMAC-SHA1"' +
            ',oauth_timestamp="' + oauth_timestamp + '"';
    if access_token <> '' then httpMsg := httpMsg +
            ',oauth_token="' + access_token + '"';
    if pin <> '' then httpMsg := httpMsg +
            ',oauth_verifier="' + pin + '"';
    httpMsg := httpMsg +
            ',oauth_version="1.0"' + ^m^j + ^m^j;

    resp := httpTransport (httpMsg, status);

    if status = 200 then
        begin
            delete (resp, 1, pos('=', resp));
            n := pos ('&', resp);
            new_access_token := copy (resp, 1, n-1);
            delete (resp, 1, pos('=', resp));
            n := pos ('&', resp);
            new_access_secret := copy (resp, 1, n-1);
            result := true;
        end
    else
        begin
            new_access_token := '';
            new_access_secret := '';
            mostraErroHTTP (resp);
            result := false;
        end;
end;

{--------------------------------------------------------}
{                Get standard browser                    }
{--------------------------------------------------------}

function GetDefaultBrowser: string;
var
    tmp : PChar;
    res : PChar;
begin
    tmp := StrAlloc(255);
    res := StrAlloc(255);
    try
        GetTempPath (255,tmp);
        FileCreate (tmp+'htmpl.htm');
        FindExecutable ('htmpl.htm',tmp,Res);
        Result := ExtractFilePath(res) + ExtractFileName(res);
        SysUtils.DeleteFile (tmp+'htmpl.htm');
    finally
        StrDispose(tmp);
        StrDispose(res);
    end;
end;

{--------------------------------------------------------}
{             start browser to authenticate              }
{--------------------------------------------------------}

procedure browseOAuthAuthenticate (temp_access_token, screen_name: string);
begin
    executaProg(GetDefaultBrowser, '',
                'https://' + oauth_site + '/oauth/authenticate' +
                '?force_login=true&screen_name=' + screen_name +
                '&oauth_token=' + temp_access_token);

    esperaProgVoltar;
end;

{--------------------------------------------------------}
{             Oauth Authentication example               }
{--------------------------------------------------------}

function getOAuthAutentication (screen_name: string;
         var oauth_token, oauth_secret: string): boolean;
var
    temp_access_token, temp_access_secret, pin: string;
begin
    getOAuthToken ('request_token', '', '', '',
                   temp_access_token, temp_access_secret);

    browseOAuthAuthenticate (temp_access_token, screen_name);

    writeln ('Digite o cdigo verificador');
    readln (pin);

    result := getOAuthToken ('access_token',
                             temp_access_token, temp_access_secret, pin,
                             oauth_token, oauth_secret);
end;

{--------------------------------------------------------}
{              Get from Twitter with Oauth               }
{--------------------------------------------------------}

function getOauthTwitterRest (Url: string; var status: integer): string;
var
    httpMsg: ansiString;
    oauth_timestamp, oauth_nonce, oauth_signature: ansiString;
begin
    oauth_timestamp := GetTimeStamp;
    oauth_nonce := GetNonce;

    oauth_signature := OauthSignature (
                 'GET', 'https://' + oauth_site + Url,
                 oauth_timestamp, oauth_nonce,
                 consumer_key, consumer_secret,
                 oauth_token, oauth_secret,
                 '', '', '');

    httpMsg :=
        'GET' + ' ' + Url + ' HTTP/1.0' + ^m^j +
        'Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*' + ^m^j +
        'Referer: ' + refURL + ^m^j +
        'Accept-Language: pt-br' + ^m^j +
        'UA-CPU: x86' + ^m^j +
        'User-Agent: TwitterVox' + ^m^j +
        'Host: api.twitter.com:443' + ^m^j +
        OauthAuthorization ('', oauth_token, oauth_signature, oauth_nonce, oauth_timestamp, '') +
         ^m^j + ^m^j;

    result := httpTransport (httpMsg, status);
end;

(*
        'Authorization: '+
            'OAuth'+
            ' oauth_consumer_key="' + consumer_key + '"' +
            ', oauth_nonce="' + oauth_nonce + '"' +
            ', oauth_signature="' + UrlEncode2(oauth_signature) + '"' +
            ', oauth_signature_method="HMAC-SHA1"' +
            ', oauth_timestamp="' + oauth_timestamp + '"' +
            ', oauth_token="' + access_key + '"' +
            ', oauth_version="1.0"' +
*)

{--------------------------------------------------------}
{              Post to Twitter with Oauth                }
{--------------------------------------------------------}

function postOauthTwitterRest (Url, PostData: string; var status: integer): string;
var
    httpMsg: ansiString;
    oauth_timestamp, oauth_nonce, oauth_signature: ansiString;
begin
    oauth_timestamp := GetTimeStamp;
    oauth_nonce := GetNonce;

    oauth_signature := OauthSignature ('POST',
                 'https://' + oauth_site + Url,
                 oauth_timestamp, oauth_nonce,
                 consumer_key, consumer_secret,
                 oauth_token, oauth_secret,
                 '', '', postData);

    httpMsg :=
        'POST ' + Url + ' HTTP/1.0' + ^m^j +
        'Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*' + ^m^j +
        'Referer: ' + refURL + ^m^j +
        'Accept-Language: pt-br' + ^m^j +
        'UA-CPU: x86' + ^m^j +
        'User-Agent: TwitterVox' + ^m^j +
        'Host: api.twitter.com:443' + ^m^j +
        'Content-Type: application/x-www-form-urlencoded' + ^m^j +
        'Content-Length: ' + intToStr(length(postData)) + ^m^j +
        OauthAuthorization ('', oauth_token, oauth_signature, oauth_nonce, oauth_timestamp, '') +
        ^m^j + ^m^j;

    httpMsg := httpMsg + postData;
    result := httpTransport (httpMsg, status);
end;

begin
    oauth_site := default_oauth_site;
    consumer_key := default_consumer_key;
    consumer_secret := default_consumer_secret;
end.

